#    Copyright 2021 Solomon M. Adams

#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at

#        http://www.apache.org/licenses/LICENSE-2.0

#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

__author__ = "Solomon M. Adams, PharmD, PhD"
__copyright__ = "Copyright 2021, Solomon M. Adams"
__license__ = "Apache v2.0"
__version__ = "0.13.4"
__maintainer__ = "Solomon M. Adams, PharmD, PhD"
__email__ = "sadams2013@gmail.com"

import logging as LOGGING

from .config import ConfigData

SPECIAL_CHROM = {
    "23": "X",
    "24": "Y"
}

# Initialize logger
LOGGING.getLogger().setLevel(LOGGING.WARNING)

def set_logging_info():
    LOGGING.getLogger().setLevel(LOGGING.INFO)

def get_config(config_path: str = None):
    """
    Set a custom config based on a path
    Args:
        config_path ([type]): path/to/config/file.ini
    """
    if config_path:
        return(ConfigData(config_path))
    else:
        return(ConfigData())





#    Copyright 2021 Solomon M. Adams

#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at

#        http://www.apache.org/licenses/LICENSE-2.0

#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import unittest
import csv
import os

import pandas as pd

from hiMoon import gene, vcf, subject, config, himoon, get_config

CONFIG = get_config()

PATH = os.path.dirname(os.path.abspath(__file__))

VCF = vcf.VarFile(PATH + "/test_files/vcf/test_samples.bcf")
CYP2D6_TABLE = PATH + "/test_files/translation_tables/CYP2D6.NC_000022.11.haplotypes.tsv"
GENE = gene.AbstractGene(CYP2D6_TABLE, vcf = VCF, config = CONFIG)
SUBJ = subject.Subject("NA12878", genes = [GENE], config = CONFIG)

class TestConfig(unittest.TestCase):

    def test_load(self):
        conf = get_config(PATH + "/test_files/config.ini")
        assert conf.CHROMOSOME_ACCESSIONS["TESTCHROM"] == "testvalue"
    
    def test_dummy_file(self):
        assert CONFIG.CHROMOSOME_ACCESSIONS["NC_000001.11"] == "1"

class TestGene(unittest.TestCase):

    def test_gene(self):
        assert GENE.gene == "CYP2D6"
    
    def test_translation_table_version(self):
        assert GENE.version == "pharmvar-4.1.6"
    
    def test_reference(self):
        assert GENE.reference == "CYP2D6(star)1"

class TestSubject(unittest.TestCase):

    def test_subject_prefix(self):
        self.assertEqual(SUBJ.prefix, "NA12878")

class TestVCF(unittest.TestCase):

    def test_samples(self):
        self.assertEqual(VCF.samples[0], "HG00111")



#    Copyright 2021 Solomon M. Adams

#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at

#        http://www.apache.org/licenses/LICENSE-2.0

#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import argparse
import os
import glob
import sys
import csv

from .subject import Subject
from .gene import AbstractGene
from .vcf import VarFile, write_variant_file, write_flat_file

from . import LOGGING, get_config, set_logging_info

def get_vcf_genes(args, CONFIG) -> ([AbstractGene], VarFile):
    """
    Prep VCF and gene objects

    Args:
        args ([type]): args

    Returns:
        Tuple
    """
    vcf = VarFile(args["vcf_file"], args["sample"], config = CONFIG)
    genes = []
    solver = args["solver"]
    if args["translation_tables"][-3:] == "tsv":
        genes.append(AbstractGene(os.path.abspath(args["translation_tables"]), vcf, solver = solver, config = CONFIG, phased = args["phased"]))
    else:
        for translation_table in glob.glob(args["translation_tables"] + "/*.tsv"):
            genes.append(AbstractGene(os.path.abspath(translation_table), vcf, solver = solver, config = CONFIG, phased = args["phased"]))
    return vcf, genes

def main() -> None:
    parser = argparse.ArgumentParser(
                        description="Match haplotypes, return raw data and/or reports.", prog="hiMoon")
    parser.add_argument("vcf_file", help="path/to/vcf file", nargs="?")
    parser.add_argument("-t", "--translation-tables",
                        help="Directory with translation tables or a single translation table file", 
                        default=None)
    parser.add_argument("-o", "--output-directory",
                        default="./",
                        help="Directory for Output Files.")
    parser.add_argument("-c", "--config-file",
                        default=None,
                        help="path to config file. -c default will write a config file that can be modified for this.")
    parser.add_argument("-i", "--loglevel-info",
                        help="Use more verbose logging output (useful for debugging).",
                        action="store_true")
    parser.add_argument("-s", "--sample",
                        help="Single sample from multisample ID (if not specified, will do all)",
                        default=None)
    parser.add_argument("-S", "--solver",
                        help="Solver to use (GLPK or CBC), default = CBC",
                        default="CBC")
    parser.add_argument("-P", "--phased",
                        help="Use phased constraint in LP",
                        action="store_true")
    
    args = vars(parser.parse_args())
    if args["config_file"] ==  "default":
        LOGGING.warning("Writing default config file and exiting")
        get_config().write_config("himoon_config.ini")
        sys.exit(0)

    if args["translation_tables"] is None:
        print("You must provide a translation table or a directory with translation tables.")
        sys.exit(1)
    if args["loglevel_info"]:
        set_logging_info()
    CONFIG = get_config(args["config_file"])
    vcf, genes = get_vcf_genes(args, CONFIG)
    subjects = [Subject(prefix = sub_id, genes = genes, config = CONFIG) for sub_id in vcf.samples]
    out_dir = args["output_directory"]
    prefix = args["vcf_file"].split("/")[-1].replace(".vcf.gz", "").replace(".bcf", "")
    write_variant_file(out_dir, subjects, prefix, genes)
    write_flat_file(out_dir, subjects, prefix)

if __name__ == "__main__": 
    main()
    


from setuptools import setup, find_packages

base = None

setup(
    name="hiMoon",
    version="0.13.4",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Healthcare Industry"
    ],
    description="Define names haplotypes from data stored in VCF",
    author="Solomon M. Adams, PharmD, PhD",
    author_email="sadams2013@gmail.com",
    packages=find_packages(),
    package_data={"hiMoon.tests": ["*"], "": ["template*.vcf"]},
    entry_points={"console_scripts": ["hiMoon = hiMoon.__main__:main"]},
    install_requires=[
        "pulp",
        "pandas",
        "pysam",
        "numpy"
    ]
)

